/*
 * Project name:
     Master;
 * Copyright:
     (c) MikroElektronika, 2005-2010.
  * Revision History:
     20110311:
       - initial release (SZ);
 * Description:
     This is a simple project which demonstrates the use of the BlueTooth Stick module.
     After establishing the connection Master repeatedly sends two strings ("mikroE" and "BlueTooth Stick")
     to the Slave. The Slave receives them and displays them at the 2x16 COG LCD.
 * Test configuration:
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/
     Oscillator:      External, 8.0000 MHz
     Ext. Modules:    BlueTooth Stick at PORTD
                      ac:BlueTooth_Stick_Board
                      http://www.mikroe.com/eng/products/view/656/bluetooth-stick/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/
 * NOTES:
  - BlueTooth Stick: Connect only Vcc, Gnd, Rx and Tx lines. You could use our Wire Jumpers:
    http://www.mikroe.com/eng/products/view/495/wire-jumpers/
    Also, 5V-3.3V Voltage Translator Board is required:
    http://www.mikroe.com/eng/products/view/182/5v-3-3v-voltage-translator-board/
  - Be sure that (near by) no BlueTooth devices other than Slave are present.
  - Turn on both modules at the same time and wait for about one minute
    until the connection between the Master and the Slave establishes.
*/



// Responses to parse
const BT_CMD  = 1;
const BT_AOK  = 2;
const BT_CONN = 3;
const BT_Done = 4;
//

// State mashine control variables
char BT_state = 0;
char response_rcvd = 0;
char responseID = 0, response = 0;
//

// Uart Rx interrupt handler
void interrupt() org IVT_ADDR_USART_RXC {
char tmp;
    tmp = UART1_Read();                         // Get received byte

    /* The responses expected from the EasyBT module:
    CMD
    AOK
    AOK
    AOK
    AOK
    AOK
    Inquiry, COD=0
    Found 1
    0006660494E3,,55AA
    Inquiry Done
    AOK
    TRYING
    MasterCONNECT*/


// Process reception through state machine
// We are parsing CMD<cr><lf>, AOK<cr><lf>, CONN and Done responses
    switch (BT_state) {
      case  0: {
                response = 0;                   // Clear response
                if (tmp == 'C')                 // We have 'C', it could be CMD<cr><lf>  or CONN
                  BT_state = 1;                 // Expecting 'M' or 'N'
                if (tmp == 'A')                 // We have 'A', it could be AOK<cr><lf>
                  BT_state = 11;                // expecting 'O'
                if (tmp == 'D')                 // We have 'D', it could be "Done"
                  BT_state = 31;                // expecting 'o'
                break;                          // ...
               }

      case  1: {
                if (tmp == 'M')
                  BT_state = 2;
                else if (tmp == 'O')
                  BT_state = 22;
                else
                  BT_state = 0;
                break;
               }


      case  2: {
                 if (tmp == 'D') {
                  response = BT_CMD;            // CMD
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }


      case 11: {
                if (tmp == 'O')
                  BT_state = 12;
                else
                  BT_state = 0;
                break;
               }

      case 12: {
                if (tmp == 'K'){
                   response = BT_AOK;
                   BT_state = 40;
                }else
                   BT_state = 0;
                break;
               }


      case 22: {
                if (tmp == 'N')
                 BT_state = 23;
                else
                  BT_state = 0;
                break;
               }

     case 23: {
                 if (tmp == 'N') {
                  response = BT_CONN;           // MasterCONNECT
                  response_rcvd = 1;
                  responseID = response;
                  }
                  BT_state = 0;
                  break;
               }

     case 31: {
                if (tmp == 'o')
                  BT_state = 32;
                else
                  BT_state = 0;
                break;
               }

      case 32: {
                if (tmp == 'n')
                 BT_state = 33;
                else
                  BT_state = 0;
                break;
               }

     case 33: {
                 if (tmp == 'e') {
                  response = BT_Done;           // "Inquiry Done"
                  response_rcvd = 1;
                  responseID = response;
                  }
                  BT_state = 0;
                  break;
               }


      case 40: {
                if (tmp == 13)
                  BT_state = 41;
                else
                  BT_state = 0;
                break;
               }


      case 41: {
                if (tmp == 10){
                response_rcvd = 1;
                responseID = response;
                }
                BT_state = 0;
                break;
                }

      default: {
                BT_state = 0;
                break;
               }
    }
}


// Get BlueTooth response, if there is any
char BT_Get_Response() {
  if (response_rcvd) {
    response_rcvd = 0;
    return responseID;
  }
  else
    return 0;
}


void main() {

// Set RTS pin to zero, we will use only RX i TX
  DDD3_bit = 0;
  PORTD3_bit = 0;

  Delay_ms(500);
  UART1_init(9600);                             // Initialize UART1 module
  Delay_ms(1000);

  SREG_I_bit  = 1;                              // Enable global interrupt
  RXCIE_bit   = 1;                              // Enable interrupt on UART receive


  do {
      UART1_Write_Text("$$$");                  // Enter command mode
      Delay_ms(500);
  } while (BT_Get_Response() != BT_CMD);


  do {
      UART1_Write_Text("SN,Bluetooth-Master");  // Name of device
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SO,Master");            // Extended status string
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SM,1");                 // Set mode (0 = slave, 1 = master, 2 = trigger, 3 = auto, 4 = DTR, 5 = ANY)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SA,1");                 // Authentication (1 to enable, 0 to disable)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART1_Write_Text("SP,mikroe");             // Security pin code (mikroe)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

   UART1_Write_Text("I,30");                    // Looks for Bluetooth devicese)
   UART1_Write(13);                             // CR
   while (BT_Get_Response() != BT_Done);

  do {
      UART1_Write_Text("SR,I");                 // Store the address just found if there is one only device found
                                                // (if there is more than one device, you need to select one by its exact address)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  UART1_Write_Text("C");                        // Connect
  UART1_Write(13);                              // CR
  while (BT_Get_Response() != BT_CONN);

  while (1) {
    UART1_Write_Text("mikroE");                 // Sending Data to Slave module
    UART1_Write(13);                            // CR
    Delay_ms(1000);
    UART1_Write_Text("BlueTooth Stick");        // Data
    UART1_Write(13);                            // CR
    Delay_ms(1000);
  }
}