/*
 * Project name:
     Slave;
 * Copyright:
     (c) MikroElektronika, 2005-2010.
  * Revision History:
     20110311:
       - initial release (SZ);
 * Description:
     This is a simple project which demonstrates the use of the BlueTooth Stick module.
     After establishing the connection Master repeatedly sends two strings ("mikroE" and "BlueTooth Stick")
     to the Slave. The Slave receives them and displays them at the 2x16 LCD.
 * Test configuration:
     MCU:             PIC16F887
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41291F.pdf#page=123
     Dev.Board:       EasyPIC6
                      http://www.mikroe.com/eng/products/view/297/easypic6-development-system/
     Oscillator:      HS, 8.00000 MHz
     Ext. Modules:    BlueTooth Stick at PORTC
                      ac:BlueTooth_Stick_Board
                      http://www.mikroe.com/eng/products/view/656/bluetooth-stick/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
  - BlueTooth Stick: Connect only Vcc, Gnd, Rx and Tx lines. You could use our Wire Jumpers:
    http://www.mikroe.com/eng/products/view/495/wire-jumpers/
    Also, 5V-3.3V Voltage Translator Board is required:
    http://www.mikroe.com/eng/products/view/182/5v-3-3v-voltage-translator-board/
  - Be sure that (near by) no BlueTooth devices other than Slave are present.
  - Turn on both modules at the same time and wait for about one minute
    until the connection between the Master and the Slave establishes.
  - Turn ON the Lcd Backlight.
*/

// responses to parse
const BT_CMD  = 1;
const BT_AOK  = 2;
const BT_CONN = 3;
const BT_END  = 4;

// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

char txt[17];
unsigned short i, tmp, DataReady;
char CMD_mode = 1;

char BT_state = 0;
char response_rcvd = 0;
char responseID = 0, response = 0;

// Uart Rx interrupt handler
void interrupt(){

  if (PIR1.RCIF == 1) {                          // Do we have uart rx interrupt request?
    tmp = UART1_Read();                          // Get received byte

  if (CMD_mode){

    /* The responses expected from the EasyBT module:
    CMD
    AOK
    AOK
    AOK
    AOK
    AOK
    END
    SlaveCONNECTmikroE
    EasyBlueTooth
    mikroE ...
    EasyBlueTooth*/


// Process reception through state machine
// We are parsing CMD<cr><lf>, AOK<cr><lf>, CONN<cr> and END<cr><lf> responses
    switch (BT_state) {
      case  0: {
                response = 0;                   // Clear response
                if (tmp == 'C')                 // We have 'C', it could be CMD<cr><lf>  or CONN
                  BT_state = 1;                 // Expecting 'M' or 'N'
                if (tmp == 'A')                 // We have 'A', it could be AOK<cr><lf>
                  BT_state = 11;                // expecting 'O'
                if (tmp == 'E')                 // We have 'E', it could be END<cr><lf>
                  BT_state = 31;                // expecting 'N'
                break;                          // ...
               }

      case  1: {
                if (tmp == 'M')
                  BT_state = 2;
                else if (tmp == 'O')
                  BT_state = 22;
                else
                  BT_state = 0;
                break;
               }


      case  2: {
                 if (tmp == 'D') {
                  response = BT_CMD;            // CMD
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }


      case 11: {
                if (tmp == 'O')
                  BT_state = 12;
                else
                  BT_state = 0;
                break;
               }

      case 12: {
                if (tmp == 'K'){
                   response = BT_AOK;           // AOK
                   BT_state = 40;
                }else
                   BT_state = 0;
                break;
               }


      case 22: {
                if (tmp == 'N')
                 BT_state = 23;
                else
                  BT_state = 0;
                break;
               }

     case 23: {
                 if (tmp == 'N') {
                  response = BT_CONN;           // SlaveCONNECTmikroE
                  response_rcvd = 1;
                  responseID = response;
                  }
                  BT_state = 0;
                  break;
               }

     case 31: {
                if (tmp == 'N')
                  BT_state = 32;
                else
                  BT_state = 0;
                 break;
               }

      case 32: {
                 if (tmp == 'D') {
                  response = BT_END;            // END
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }

      case 40: {
                if (tmp == 13)
                  BT_state = 41;
                else
                  BT_state = 0;
                break;
               }


      case 41: {
                if (tmp == 10){
                response_rcvd = 1;
                responseID = response;
                }
                BT_state = 0;
                break;
                }

      default: {
                BT_state = 0;
                break;
               }
    }
  }
  else {
       if (tmp == 13) {
         txt[i] = 0;                            // Puting 0 at the end of the string
         DataReady = 1;                         // Data is received
                   }
     else {
      txt[i] = tmp;                             // Moving the data received from UART to string txt[]
      i++;
     }
   PIR1.RCIF = 0;
   }

  }
}

// Get BlueTooth response, if there is any
char BT_Get_Response() {
  if (response_rcvd) {
    response_rcvd = 0;
    return responseID;
  }
  else
    return 0;
}


void main() {
  ANSEL  = 0;                                   // Configure AN pins as digital I/O
  ANSELH = 0;
  C1ON_bit = 0;                                 // Disable comparators
  C2ON_bit = 0;

// Set RTS pin to zero, we will use only RX i TX
  TRISC3_bit = 0;
  RC3_bit = 0;

// Enable uart rx interrupt
  PIE1.RCIE   = 1;
  INTCON.PEIE = 1;
  INTCON.GIE  = 1;

  LCD_Init();                                   // LCD Init
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  Lcd_Out(1,1,"Connecting!");
  Lcd_Out(2,1,"Please, wait...");

  UART1_init(9600);                             // Initialize UART1 module
  Delay_ms(1500);


  do {
      UART1_Write_Text("$$$");                  // Enter command mode
      Delay_ms(500);
  } while (BT_Get_Response() != BT_CMD);

  do {
      UART1_Write_Text("SN,Bluetooth-Slave");   // Name of device
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

   do {
      UART1_Write_Text("SO,Slave");             // Extended status string
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SM,0");                 // Set mode (0 = slave, 1 = master, 2 = trigger, 3 = auto, 4 = DTR, 5 = ANY)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SA,1");                 // Authentication (1 to enable, 0 to disable)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART1_Write_Text("SP,mikroe");             // Security pin code (mikroe)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART1_Write_Text("---");                   // Security pin code (mikroe)
     UART1_Write(13);                           // CR
     Delay_ms(500);
  } while (BT_Get_Response() != BT_END);

  while (BT_Get_Response() != BT_CONN);

  LCD_Cmd(_LCD_CLEAR);

  CMD_mode = 0;
  INTCON.GIE  = 0;

  DataReady = 0;

  Lcd_Out(1,1,"Connected!");
  Delay_ms(1000);
  LCD_Cmd(_LCD_CLEAR);                          // Clear display
  Lcd_Out(1,1,"Receiving...");

  while (1) {
    tmp = 0;                                    // Init variables
    i = 0;
    memset(txt, 0, 17);                         // Clear array of chars

    INTCON.GIE  = 1;                            // Interrupts allowed

    while (!DataReady) {                        // Wait while the data is received
        ;
    }

    INTCON.GIE  = 0;                            // Interrupts forbiden
    DataReady = 0;

    LCD_Cmd(_LCD_CLEAR);                        // Clear display
    Lcd_Out(1,1,"Received:");
    Lcd_Cmd(_LCD_SECOND_ROW);
    i = 0;
    while (txt[i] != 0){

       Lcd_Chr_CP(txt[i]);                      // Displaying the received text on the LCD
       i++;                                     // There is delay before the data is displayed
    }                                           // This is due the init duration and synchronization of two Zigbit modules

  }
}